@@ -31,6 +31,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/mman.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -46,10 +47,142 @@
 #include <lal/LALCache.h>
 
 
-#include <gstlal_debug.h>
+#include <gstlal/gstlal_debug.h>
 #include <gstlal_cachesrc.h>
 
 
+/*
+ * ========================================================================
+ *
+ *                               GstURIHandler
+ *
+ * ========================================================================
+ */
+
+
+#define URI_SCHEME "lalcache"
+
+
+static GstURIType uri_get_type(GType type)
+{
+	return GST_URI_SRC;
+}
+
+
+/* 1.0:  this becomes static const gchar *const * */
+static gchar **uri_get_protocols(GType type)
+{
+	/* 1.0:  this becomes
+	static const gchar *protocols[] = {URI_SCHEME, NULL};
+	*/
+	static gchar *protocols[] = {(gchar *) URI_SCHEME, NULL};
+
+	return protocols;
+}
+
+
+/* 1.0:  this becomes static gchar * */
+static const gchar *uri_get_uri(GstURIHandler *handler)
+{
+	GstLALCacheSrc *element = GSTLAL_CACHESRC(handler);
+	GString *uri = g_string_new(URI_SCHEME "://");
+	gchar separator = '?';
+
+	g_string_append_uri_escaped(uri, element->location, NULL, FALSE);
+
+	if(element->cache_src_regex) {
+		g_string_append_c(uri, separator);
+		g_string_append(uri, "cache-src-regex=");
+		g_string_append_uri_escaped(uri, element->cache_src_regex, NULL, FALSE);
+		separator = '&';
+	}
+
+	if(element->cache_dsc_regex) {
+		g_string_append_c(uri, separator);
+		g_string_append(uri, "cache-dsc-regex=");
+		g_string_append_uri_escaped(uri, element->cache_dsc_regex, NULL, FALSE);
+	}
+
+	/* 1.0:  returning this value won't be a memory leak */
+	return g_string_free(uri, FALSE);
+}
+
+
+static gchar *g_uri_unescape_string_inplace(gchar **s)
+{
+	gchar *unescaped = g_uri_unescape_string(*s, NULL);
+	g_free(*s);
+	*s = unescaped;
+	return *s;
+}
+
+
+/* 1.0:  this gets a GError ** argument */
+static gboolean uri_set_uri(GstURIHandler *handler, const gchar *uri)
+{
+	GstLALCacheSrc *element = GSTLAL_CACHESRC(handler);
+	gchar *scheme = g_uri_parse_scheme(uri);
+	int query_offset;
+	gchar *location = NULL;
+	gchar *cache_src_regex = NULL;
+	gchar *cache_dsc_regex = NULL;
+	gboolean success = TRUE;
+
+	success = !strcmp(scheme, URI_SCHEME);
+	if(success) {
+		success &= sscanf(uri, URI_SCHEME "://%m[^?]%n", &location, &query_offset) >= 1;
+		if(!success)
+			GST_ERROR_OBJECT(element, "bad uri '%s'", uri);
+	} else
+		GST_ERROR_OBJECT(element, "wrong scheme '%s'", scheme);
+	if(success && uri[query_offset] == '?') {
+		gchar **fragments = g_strsplit(&uri[++query_offset], "&", 0);
+		gchar **fragment;
+		for(fragment = fragments; *fragment; fragment++) {
+			gchar **namevalue = g_strsplit(*fragment, "=", 2);
+			success &= namevalue[0] && namevalue[1];
+			if(success) {
+				g_uri_unescape_string_inplace(&namevalue[0]);
+				g_uri_unescape_string_inplace(&namevalue[1]);
+				if(!g_strcmp0("cache-src-regex", namevalue[0])) {
+					g_free(cache_src_regex);
+					cache_src_regex = g_strdup(namevalue[1]);
+				} else if(!g_strcmp0("cache-dsc-regex", namevalue[0])) {
+					g_free(cache_dsc_regex);
+					cache_dsc_regex = g_strdup(namevalue[1]);
+				} else {
+					GST_ERROR_OBJECT(element, "query '%s' not recognized", namevalue[0]);
+					success = FALSE;
+				}
+			} else
+				GST_ERROR_OBJECT(element, "invalid query '%s'", *fragment);
+			g_strfreev(namevalue);
+		}
+		g_strfreev(fragments);
+	}
+	if(success)
+		g_object_set(G_OBJECT(element), "location", g_uri_unescape_string_inplace(&location), "cache-src-regex", cache_src_regex, "cache-dsc-regex", cache_dsc_regex, NULL);
+
+	g_free(location);
+	g_free(cache_src_regex);
+	g_free(cache_dsc_regex);
+	g_free(scheme);
+	return success;
+}
+
+
+static void uri_handler_init(gpointer g_iface, gpointer iface_data)
+{
+	GstURIHandlerInterface *iface = (GstURIHandlerInterface *) g_iface;
+
+	iface->get_uri = GST_DEBUG_FUNCPTR(uri_get_uri);
+	iface->set_uri = GST_DEBUG_FUNCPTR(uri_set_uri);
+	/* 1.0:  this is ->get_type */
+	iface->get_type_full = GST_DEBUG_FUNCPTR(uri_get_type);
+	/* 1.0:  this is ->get_protocols */
+	iface->get_protocols_full = GST_DEBUG_FUNCPTR(uri_get_protocols);
+}
+
 
 /*
  * ============================================================================
@@ -66,6 +199,13 @@ GST_DEBUG_CATEGORY_STATIC(GST_CAT_DEFAULT);
 
 static void additional_initializations(GType type)
 {
+	static const GInterfaceInfo uri_handler_info = {
+		uri_handler_init,
+		NULL,
+		NULL
+	};
+	g_type_add_interface_static(type, GST_TYPE_URI_HANDLER, &uri_handler_info);
+
 	GST_DEBUG_CATEGORY_INIT(GST_CAT_DEFAULT, "lal_cachesrc", 0, "lal_cachesrc element");
 }
 
@@ -106,13 +246,14 @@ static GstFlowReturn read_buffer(GstBaseSrc *basesrc, const char *path, int fd,
 	result = gst_pad_alloc_buffer(pad, offset, size, GST_PAD_CAPS(pad), buf);
 	if(result != GST_FLOW_OK)
 		goto done;
+	g_assert_cmpuint(GST_BUFFER_OFFSET(*buf), ==, offset);
 	g_assert_cmpuint(GST_BUFFER_SIZE(*buf), ==, size);
 
 	read_offset = 0;
 	do {
 		ssize_t bytes_read = read(fd, GST_BUFFER_DATA(*buf) + read_offset, GST_BUFFER_SIZE(*buf) - read_offset);
 		if(bytes_read < 0) {
-			GST_ELEMENT_ERROR(basesrc, RESOURCE, READ, (NULL), ("read('%s') failed: %s", path, sys_errlist[errno]));
+			GST_ELEMENT_ERROR(basesrc, RESOURCE, READ, (NULL), ("read('%s') failed: %s", path, strerror(errno)));
 			gst_buffer_unref(*buf);
 			*buf = NULL;
 			result = GST_FLOW_ERROR;
@@ -122,6 +263,8 @@ static GstFlowReturn read_buffer(GstBaseSrc *basesrc, const char *path, int fd,
 	} while(read_offset < size);
 	g_assert_cmpuint(read_offset, ==, size);
 
+	GST_BUFFER_OFFSET_END(*buf) = offset + size;
+
 done:
 	return result;
 }
@@ -148,7 +291,7 @@ static GstFlowReturn mmap_buffer(GstBaseSrc *basesrc, const char *path, int fd,
 	GST_BUFFER_FLAG_SET(*buf, GST_BUFFER_FLAG_READONLY);
 	GST_BUFFER_DATA(*buf) = mmap(NULL, size, PROT_READ, MAP_SHARED, fd, 0);
 	if(!GST_BUFFER_DATA(*buf)) {
-		GST_ELEMENT_ERROR(basesrc, RESOURCE, READ, (NULL), ("mmap('%s') failed: %s", path, sys_errlist[errno]));
+		GST_ELEMENT_ERROR(basesrc, RESOURCE, READ, (NULL), ("mmap('%s') failed: %s", path, strerror(errno)));
 		gst_buffer_unref(*buf);
 		*buf = NULL;
 		result = GST_FLOW_ERROR;
@@ -156,17 +299,14 @@ static GstFlowReturn mmap_buffer(GstBaseSrc *basesrc, const char *path, int fd,
 	}
 	GST_BUFFER_SIZE(*buf) = size;
 	GST_BUFFER_OFFSET(*buf) = offset;
+	GST_BUFFER_OFFSET_END(*buf) = offset + size;
 	gst_buffer_set_caps(*buf, GST_PAD_CAPS(GST_BASE_SRC_PAD(basesrc)));
 
 	/*
 	 * hack to get both the data pointer and the size to the munmap()
 	 * call.  the mallocdata pointer is set to the buffer object
 	 * itself, and the freefunc looks inside to get the real pointer
-	 * and the size.  this probably goes really wrong if a subbuffer is
-	 * ever made from the buffer, but then the application can choose
-	 * not to use mmap()ed buffers if it wants to be able to do
-	 * subbuffering.  for the .gwf frame file use case, subbuffering is
-	 * (mostly) nonsensical anyway.
+	 * and the size.
 	 */
 
 	GST_BUFFER_MALLOCDATA(*buf) = (void *) *buf;
@@ -207,12 +347,9 @@ static guint time_to_index(GstLALCacheSrc *element, GstClockTime t)
 	 * time
 	 */
 
-	for(i = 0; i < element->cache->length; i++) {
-		if(i)
-			g_assert_cmpuint(cache_entry_start_time(element, i), >=, cache_entry_start_time(element, i - 1));
+	for(i = 0; i < element->cache->length; i++)
 		if(cache_entry_end_time(element, i) > t)
 			break;
-	}
 
 	return i;
 }
@@ -256,7 +393,9 @@ static gboolean start(GstBaseSrc *basesrc)
 	}
 	GST_DEBUG_OBJECT(element, "%d item(s) remain in cache after sieve", element->cache->length);
 
-	if(XLALCacheSort(element->cache)) {
+	if(!element->cache->length)
+		GST_WARNING_OBJECT(element, "cache is empty!");
+	else if(XLALCacheSort(element->cache)) {
 		GST_ELEMENT_ERROR(element, LIBRARY, FAILED, (NULL), ("error sorting cache '%s': %s", element->location, XLALErrorString(XLALGetBaseErrno())));
 		XLALClearErrno();
 		XLALDestroyCache(element->cache);
@@ -338,13 +477,13 @@ static GstFlowReturn create(GstBaseSrc *basesrc, guint64 offset, guint size, Gst
 
 	fd = open(path, O_RDONLY);
 	if(fd < 0) {
-		GST_ELEMENT_ERROR(element, RESOURCE, READ, (NULL), ("open('%s') failed: %s", path, sys_errlist[errno]));
+		GST_ELEMENT_ERROR(element, RESOURCE, READ, (NULL), ("open('%s') failed: %s", path, strerror(errno)));
 		result = GST_FLOW_ERROR;
 		goto done;
 	}
 
 	if(fstat(fd, &statinfo)) {
-		GST_ELEMENT_ERROR(element, RESOURCE, READ, (NULL), ("fstat('%s') failed: %s", path, sys_errlist[errno]));
+		GST_ELEMENT_ERROR(element, RESOURCE, READ, (NULL), ("fstat('%s') failed: %s", path, strerror(errno)));
 		result = GST_FLOW_ERROR;
 		close(fd);
 		goto done;
@@ -358,12 +497,15 @@ static GstFlowReturn create(GstBaseSrc *basesrc, guint64 offset, guint size, Gst
 	if(result != GST_FLOW_OK)
 		goto done;
 
+	/*
+	 * finish setting buffer metadata.  need_discont is TRUE for the
+	 * first buffer, so cache_entry_end_time() won't be invoked with an
+	 * index of -1
+	 */
+
 	GST_BUFFER_TIMESTAMP(*buf) = cache_entry_start_time(element, element->index);
 	GST_BUFFER_DURATION(*buf) = cache_entry_duration(element, element->index);
-	GST_BUFFER_OFFSET_END(*buf) = GST_BUFFER_OFFSET(*buf) + GST_BUFFER_SIZE(*buf);
-	basesrc->offset += GST_BUFFER_SIZE(*buf);
-	/* need_discont is TRUE for the first buffer, so
-	 * cache_entry_end_time() won't be invoked with an index of -1 */
+	basesrc->offset = GST_BUFFER_OFFSET_END(*buf);
 	if(element->need_discont || GST_BUFFER_TIMESTAMP(*buf) != cache_entry_end_time(element, element->index - 1)) {
 		GST_BUFFER_FLAG_SET(*buf, GST_BUFFER_FLAG_DISCONT);
 		element->need_discont = FALSE;
