@@ -54,7 +54,7 @@
  */
 
 
-#include <gstlal_debug.h>
+#include <gstlal/gstlal_debug.h>
 #include <gstlal_gate.h>
 
 
