@@ -72,8 +72,8 @@
  */
 
 
-#include <gstlal.h>
-#include <gstlal_tags.h>
+#include <gstlal/gstlal.h>
+#include <gstlal/gstlal_tags.h>
 #include <gstlal_simulation.h>
 
 
@@ -334,7 +334,7 @@ static int update_simulation_series(REAL8TimeSeries *h, GSTLALSimulation *elemen
 	 * get detector information
 	 */
 
-	detector = XLALInstrumentNameToLALDetector(h->name);
+	detector = XLALDetectorPrefixToLALDetector(h->name);
 	if(!detector)
 		XLAL_ERROR(XLAL_EFUNC);
 
