@@ -59,6 +59,12 @@ from gstlal import pipeio
 from gstlal import simplehandler
 
 
+## @file
+# module for reference psds
+
+## @package python.reference_psd
+# the reference_psd module
+
 #
 # =============================================================================
 #
@@ -89,7 +95,30 @@ class PSDHandler(simplehandler.Handler):
 # measure_psd()
 #
 
-
+## A pipeline to measure a PSD
+#
+# @dot
+# digraph G {
+#	// graph properties
+#
+#	rankdir=LR;
+#	compound=true;
+#	node [shape=record fontsize=10 fontname="Verdana"];
+#	edge [fontsize=8 fontname="Verdana"];
+#
+#	// nodes
+#
+#	"mkbasicsrc()" [URL="\ref datasource.mkbasicsrc()"];
+#	capsfilter1 [URL="\ref pipeparts.mkcapsfilter()"];
+#	resample [URL="\ref pipeparts.mkresample()"];
+#	capsfilter2  [URL="\ref pipeparts.mkcapsfilter()"];
+#	queue [URL="\ref pipeparts.mkqueue()"];
+#	whiten [URL="\ref pipeparts.mkwhiten()"];
+#	fakesink [URL="\ref pipeparts.mkfakesink()"];
+#
+#	"mkbasicsrc()" -> capsfilter1 -> resample -> capsfilter2 -> queue -> whiten -> fakesink;
+# } 
+# @enddot
 def measure_psd(gw_data_source_info, instrument, rate, psd_fft_length = 8, verbose = False):
 	#
 	# 8 FFT-lengths is just a ball-parky estimate of how much data is
@@ -97,7 +126,7 @@ def measure_psd(gw_data_source_info, instrument, rate, psd_fft_length = 8, verbo
 	# code requires a minimum of 1)
 	#
 
-	if float(abs(gw_data_source_info.seg)) < 8 * psd_fft_length:
+	if gw_data_source_info.seg is not None and float(abs(gw_data_source_info.seg)) < 8 * psd_fft_length:
 		raise ValueError("segment %s too short" % str(gw_data_source_info.seg))
 
 	#
@@ -116,33 +145,20 @@ def measure_psd(gw_data_source_info, instrument, rate, psd_fft_length = 8, verbo
 	head = pipeparts.mkresample(pipeline, head, quality = 9)
 	head = pipeparts.mkcapsfilter(pipeline, head, "audio/x-raw-float, rate=%d" % rate)
 	head = pipeparts.mkqueue(pipeline, head, max_size_buffers = 8)
-	head = pipeparts.mkwhiten(pipeline, head, psd_mode = 0, zero_pad = 0, fft_length = psd_fft_length, average_samples = int(round(float(abs(gw_data_source_info.seg)) / (psd_fft_length / 2) - 1)), median_samples = 7)
+	if gw_data_source_info.seg is not None:
+		average_samples = int(round(float(abs(gw_data_source_info.seg)) / (psd_fft_length / 2) - 1))
+	else:
+		#FIXME maybe let the user specify this
+		average_samples = 64
+	head = pipeparts.mkwhiten(pipeline, head, psd_mode = 0, zero_pad = 0, fft_length = psd_fft_length, average_samples = average_samples, median_samples = 7)
 	pipeparts.mkfakesink(pipeline, head)
 
 	#
-	# setup signal handler to shutdown pipeline
+	# setup signal handler to shutdown pipeline for live data
 	#
 
-	class OneTimeSignalHandler(object):
-		def __init__(self, pipeline):
-			self.pipeline = pipeline
-			self.count = 0
-
-		def __call__(self, signum, frame):
-			self.count += 1
-			if self.count == 1:
-				print >>sys.stderr, "*** SIG %d attempting graceful shutdown (this might take several minutes) ... ***" % signum
-				try:
-					if not self.pipeline.send_event(gst.event_new_eos()):
-						raise Exception("pipeline.send_event(EOS) returned failure")
-				except Exception, e:
-					print >>sys.stderr, "graceful shutdown failed: %s\naborting." % str(e)
-					os._exit(1)
-			else:
-				print >>sys.stderr, "*** received SIG %d %d times... ***" % (signum, self.count)
-
-	signal.signal(signal.SIGINT, OneTimeSignalHandler(pipeline))
-	signal.signal(signal.SIGTERM, OneTimeSignalHandler(pipeline))
+	if gw_data_source_info.data_source in ("lvshm", "framexmit"):# FIXME what about nds online?
+		simplehandler.OneTimeSignalHandler(pipeline)
 
 	#
 	# process segment
@@ -592,3 +608,41 @@ def interpolate_psd(psd, deltaF):
 		sampleUnits = psd.sampleUnits,
 		data = psd_data
 	)
+
+
+def movingmedian(psd, window_size):
+	data = numpy.copy(psd.data)
+	for i in range(window_size, len(data)-window_size):
+		data[i] = numpy.median(data[i-window_size:i+window_size])
+	return laltypes.REAL8FrequencySeries(
+		name = psd.name,
+		epoch = psd.epoch,
+		f0 = psd.f0,
+		deltaF = psd.deltaF,
+		sampleUnits = psd.sampleUnits,
+		data = data
+	)
+
+
+def polyfit(psd, minsample, maxsample, order, verbose = False):
+	# f / f_min between f_min and f_max, i.e. f[0] here is 1
+	f = numpy.arange(maxsample - minsample) * psd.deltaF + 1
+	data = psd.data[minsample:maxsample]
+
+	logf = numpy.linspace(numpy.log(f[0]), numpy.log(f[-1]), 100000)
+	interp = interpolate.interp1d(numpy.log(f), numpy.log(data))
+	data = interp(logf)
+	p = numpy.poly1d(numpy.polyfit(logf, data, order))
+	if verbose:
+		print >> sys.stderr, "\nFit polynomial is: \n\nlog(PSD) = \n", p, "\n\nwhere x = f / f_min\n"
+	data = numpy.exp(p(numpy.log(f)))
+	olddata = psd.data
+	olddata[minsample:maxsample] = data
+	return laltypes.REAL8FrequencySeries(
+		name = psd.name,
+		epoch = psd.epoch,
+		f0 = psd.f0,
+		deltaF = psd.deltaF,
+		sampleUnits = psd.sampleUnits,
+		data = olddata
+	)
